# Getting started of SEMuP

## Repository description
* `case_studies`: directory expected to contain case study data and output
* `containers`: directory containing virtual machine images and its environment setting
* `ktest_to_unittest`: pipeline codes for generating unit test
* `pre_semu`: pipeline codes for `pre_semu`, it provides information for `SEMu`
* `underlying_test_generation`: pipeline codes for ??
* `HPC`: scripts for executing parallel jobs on HPC
* `Vagrantfile`: configuration file for the vagrant box (for Windows and Mac OS users)


## Pre-requisition
We use Singularity to provide the same environment for all the users.
Users who work on Linux operating systems can install Singularity directly on their machines.
But Windows and Mac OS users need to rely on a Linux virtual machine since Singularity only supports Linux.
SyLabs, which has developed Singularity, provides Vagrant boxes with Singularity pre-installed on Linux.
We recommend you install Vagrant.
For the installation, please follow the guidelines from [the official website](https://docs.sylabs.io/guides/3.8/admin-guide/installation.html).

Note that we used Singularity 3.8 CE version.



## Connecting to vagrant box (for Windows or Mac OS users)
* We provide all the configuration to set up the vagrant instance for Singularity in `Vagrantfile`
* Please use the commands below in the root repository of MOTIF
```shell
# The command below creates a virtual machine instance according to the Vagrantfile based on the root repository.
# This will bind automatically the root repository to the directory /vagrant inside of the container
$ vagrant up 
# Connect into the vagrant box instance
$ vagrant ssh
# Move to the bound directory, which is sharing between vagrant container and the host OS
[vagrant]$ cd /vagrant 
```
> Now, we assume that you are in the directory /vagrant in the vagrant instance, which is the same as the root repository of the SEMuP.


## Preparing singularity container
* We provide a singularity container that we used for our experiments.
* Download it using the command below and put them in the `containers` directory
* If you want to build the singularity image, please follow the guideline in [Appendix](#Appendix)
```shell
# We also provide pre-built singularity image in the Figshare webpage.
# You can just download it into `containers` directory using the following command:
$ wget -O containers/semup_full.sif https://figshare.com/ndownloader/files/40458779
```


## Preparing a target subject
We provide the subject data for ASN1. It contains additional scripts, software under test, target live mutants, and configuration files for SEMuP.
The commands below will download the data and extract them into the `case_studies` directory.
```shell
$ wget -O case_studies/ASN1_SEMuP.tar https://figshare.com/ndownloader/files/41974677
$ tar xf case_studies/ASN1_SEMuP.tar -C case_studies/
```


## Executing SEMuP for ASN1
```shell
# We assume that you are in the root repository of SEMuP
# Move to the subject directory
$ cd case_studies/ASN1

# Generating test drivers (in a singularity container)
#   - Connecting to the singularity image
case_studies/ASN1$ scripts/shell.sh    # singularity shell --bind ../../:/home/SEMUP -H /home/SEMUP ../../containers/semup_full.sif   
#   - Moving to the subject directory
[singularity]$ cd case_studies/ASN1
#   - Generating test drivers
[singularity]case_studies/ASN1$ ./util_codes/generate_direct.py ./WORKSPACE/DOWNLOADED/repos/test.c util_codes/direct-test " -I./WORKSPACE/DOWNLOADED/repos/" -c ./util_codes/generate_template_config.json


# Generating mutants with `meta-mu` information (outside of Singularity)
case_studies/ASN1$ ./scripts/run_singularity.sh --env ENV_SEMUP_SRC_FILE=./test.c --env ENV_SEMUP_STOP=TRUE mutation WORKSPACE/DOWNLOADED/live_mutants WORKSPACE/OUTPUT/live_mutants_output
# Executing SEMu (outside of a singularity container)
case_studies/ASN1$ ./scripts/run_local.sh presemu ./WORKSPACE/DOWNLOADED/live_mutants ./WORKSPACE/_expLocal


# Making execution results
case_studies/ASN1$ ./scripts/make_summary.py ./WORKSPACE/_expLocal
#  -- This command will generate the results file summary.csv in the `./WORKSPACE/_exp01`
```




# Appendix
## MASS compilation
> SEMuP relies on the MASS to generate mutants.
```shell
# Build singularity image for mass
$ cd SEMUP/containers
SEMUP/containers$ sudo singularity build  mass.sif singularity_mass.def

# Download MASS source code
SEMUP/containers$ git clone https://github.com/SNTSVV/FAQAS_MASS MASS

# Build MASS   
#   - it will download llvm-3.8.1, which is old version so we need to compile it by ourselves  
SEMUP/containers$ singularity exec --bind ./MASS:/expr -H /expr mass.sif ./llvm-build.sh

```

## Building singularity image for SEMuP
> Originally, SEMu works on a docker (thierrytct/klee-semu:llvm-3.4.2), we convert it to the singularity image.
```shell
# the following singularity works currently (need to check the last part)
$ cd containers

# It requires minium 20GB disk size of vagrant instance
#  - This command will copy the compiled MASS source code into the singularity image
containers$ sudo singularity build semup_full.sif singularity_semup_full.def

# Test the singularity
containers$ cd ..
$ singularity shell --bind ./:/home/SEMUP -H /home/SEMUP semup_full.sif
$ singularity exec --bind ./:/home/SEMUP -H /home/SEMUP semup_full.sif pwd
```